"""Constant variables used across modules."""

BASE_URL = "https://api.nb.no/dhlab"
NGRAM_API = "https://api.nb.no/dhlab/nb_ngram/ngram/query"
GALAXY_API = "https://api.nb.no/dhlab/nb_ngram_galaxies/galaxies/query"
